==> dynamic_links-1.0.0.tar.gz: dynamic_links/src/Entity/DynamicLink.php <==
<?php

namespace Drupal\dynamic_links\Entity;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\dynamic_links\DynamicLinkInterface;

/**
 * Defines the dynamic link entity type.
 *
 * @ConfigEntityType(
 *   id = "dynamic_link",
 *   label = @Translation("Dynamic link"),
 *   label_collection = @Translation("Dynamic links"),
 *   label_singular = @Translation("dynamic link"),
 *   label_plural = @Translation("dynamic links"),
 *   label_count = @PluralTranslation(
 *     singular = "@count dynamic link",
 *     plural = "@count dynamic links",
 *   ),
 *   handlers = {
 *     "list_builder" = "Drupal\dynamic_links\DynamicLinkListBuilder",
 *     "route_provider" = {
 *        "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *      },
 *     "form" = {
 *       "add" = "Drupal\dynamic_links\Form\DynamicLinkForm",
 *       "edit" = "Drupal\dynamic_links\Form\DynamicLinkForm",
 *       "delete" = "Drupal\Core\Entity\EntityDeleteForm",
 *     },
 *   },
 *   config_prefix = "link",
 *   admin_permission = "administer dynamic_link",
 *   links = {
 *     "collection" = "/admin/structure/dynamic-link",
 *     "add-form" = "/admin/structure/dynamic-link/add",
 *     "edit-form" = "/admin/structure/dynamic-link/{dynamic_link}",
 *     "delete-form" = "/admin/structure/dynamic-link/{dynamic_link}/delete",
 *   },
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "id",
 *   },
 *   config_export = {
 *     "id",
 *     "url",
 *     "redirects",
 *     "routes",
 *   },
 * )
 */
class DynamicLink extends ConfigEntityBase implements DynamicLinkInterface {

  /**
   * An array of URLs and cache metadata, keyed by user ID.
   *
   * This is necessary to prevent re-building and checking
   * links after checking access in "_custom_access".
   *
   * @var array
   */
  protected array $requestCache = [];

  /**
   * The link ID.
   */
  protected string $id;

  /**
   * The link URL.
   */
  protected string $url = '';

  /**
   * An array of redirect URLs.
   */
  protected array $redirects = [];

  /**
   * An array of redirect routes.
   */
  protected ?array $routes = NULL;

  /**
   * {@inheritdoc}
   */
  public function getUrl(): string {
    return $this->url;
  }

  /**
   * {@inheritdoc}
   */
  public function getRedirects(): array {
    return $this->redirects;
  }

  /**
   * {@inheritdoc}
   */
  public function getRoutes(): ?array {
    return $this->routes;
  }

  /**
   * {@inheritdoc}
   */
  public function getFirstAvailable(AccountInterface $account): array {
    $uid = $account->id();
    if (isset($this->requestCache[$uid])) {
      return $this->requestCache[$uid];
    }

    $available_url = NULL;
    $cacheability = new CacheableMetadata();
    $cacheability->addCacheContexts(['user']);
    if ($routes = $this->getRoutes()) {
      foreach ($routes as $route) {
        $this->checkUrl(Url::fromRoute($route['name'], $route['parameters'], $route['options']), $account, $cacheability, $available_url,);
      }
    }
    else {
      foreach ($this->getRedirects() as $url) {
        $this->checkUrl(Url::fromUserInput($url), $account, $cacheability, $available_url);
      }
    }

    $result = [
      'url' => $available_url,
      'cacheability' => $cacheability,
    ];
    $this->requestCache[$uid] = $result;
    return $result;
  }

  /**
   * Checks if the given URL is accessible to the given user.
   */
  protected function checkUrl(Url $url, AccountInterface $account, CacheableMetadata $cacheability, ?Url &$available_url): void {
    $access = $url->access($account, TRUE);
    $cacheability->addCacheableDependency($access);
    if (!$available_url && $access->isAllowed()) {
      $available_url = $url;
    }
  }

}

==> dynamic_links-1.0.3.tar.gz: dynamic_links/src/Entity/DynamicLink.php <==
<?php

namespace Drupal\dynamic_links\Entity;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\dynamic_links\DynamicLinkInterface;

/**
 * Defines the dynamic link entity type.
 *
 * @ConfigEntityType(
 *   id = "dynamic_link",
 *   label = @Translation("Dynamic link"),
 *   label_collection = @Translation("Dynamic links"),
 *   label_singular = @Translation("dynamic link"),
 *   label_plural = @Translation("dynamic links"),
 *   label_count = @PluralTranslation(
 *     singular = "@count dynamic link",
 *     plural = "@count dynamic links",
 *   ),
 *   handlers = {
 *     "list_builder" = "Drupal\dynamic_links\DynamicLinkListBuilder",
 *     "route_provider" = {
 *        "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *      },
 *     "form" = {
 *       "add" = "Drupal\dynamic_links\Form\DynamicLinkForm",
 *       "edit" = "Drupal\dynamic_links\Form\DynamicLinkForm",
 *       "delete" = "Drupal\Core\Entity\EntityDeleteForm",
 *     },
 *   },
 *   config_prefix = "link",
 *   admin_permission = "administer dynamic_link",
 *   links = {
 *     "collection" = "/admin/structure/dynamic-link",
 *     "add-form" = "/admin/structure/dynamic-link/add",
 *     "edit-form" = "/admin/structure/dynamic-link/{dynamic_link}",
 *     "delete-form" = "/admin/structure/dynamic-link/{dynamic_link}/delete",
 *   },
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "id",
 *   },
 *   config_export = {
 *     "id",
 *     "url",
 *     "redirects",
 *     "routes",
 *   },
 * )
 */
class DynamicLink extends ConfigEntityBase implements DynamicLinkInterface {

  /**
   * An array of URLs and cache metadata, keyed by user ID.
   *
   * This is necessary to prevent re-building and checking
   * links after checking access in "_custom_access".
   */
  protected array $requestCache = [];

  /**
   * The link ID.
   */
  protected string $id;

  /**
   * The link URL.
   */
  protected string $url = '';

  /**
   * An array of redirect URLs.
   */
  protected array $redirects = [];

  /**
   * An array of redirect routes.
   */
  protected ?array $routes = NULL;

  /**
   * {@inheritdoc}
   */
  public function getUrl(): string {
    return $this->url;
  }

  /**
   * {@inheritdoc}
   */
  public function getRedirects(): array {
    return $this->redirects;
  }

  /**
   * {@inheritdoc}
   */
  public function getRoutes(): ?array {
    return $this->routes;
  }

  /**
   * {@inheritdoc}
   */
  public function getFirstAvailable(AccountInterface $account): array {
    $uid = $account->id();
    if (isset($this->requestCache[$uid])) {
      return $this->requestCache[$uid];
    }

    $available_url = NULL;
    $cacheability = new CacheableMetadata();
    if ($routes = $this->getRoutes()) {
      foreach ($routes as $route) {
        $this->checkUrl(Url::fromRoute($route['name'], $route['parameters'], $route['options']), $account, $cacheability, $available_url);
      }
    }
    else {
      foreach ($this->getRedirects() as $url) {
        $this->checkUrl(Url::fromUserInput($url), $account, $cacheability, $available_url);
      }
    }

    $result = [
      'url' => $available_url,
      'cacheability' => $cacheability,
    ];
    $this->requestCache[$uid] = $result;
    return $result;
  }

  /**
   * Checks if the given URL is accessible to the given user.
   */
  protected function checkUrl(Url $url, AccountInterface $account, CacheableMetadata $cacheability, ?Url &$available_url): void {
    $access = $url->access($account, TRUE);
    $cacheability->addCacheableDependency($access);
    if (!$available_url && $access->isAllowed()) {
      $available_url = $url;
    }
  }

}
